package com.engine.thinktrans.service.impl;

import com.engine.core.impl.Service;
import com.engine.thinktrans.entity.PersonnelSizeTop;
import com.engine.thinktrans.entity.ReportChildrenData;
import com.engine.thinktrans.entity.ReportColumns;
import com.engine.thinktrans.entity.param.PersonnelSizeTopParam;
import com.engine.thinktrans.entity.param.TendencyReportParam;
import com.engine.thinktrans.entity.vo.TendencyReportVO;
import com.engine.thinktrans.service.ReportCollectService;
import com.engine.thinktrans.util.CommonDateUtil;
import com.weaver.general.Util;
import lombok.SneakyThrows;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.hrm.company.DepartmentComInfo;
import weaver.hrm.company.SubCompanyComInfo;

import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.Year;
import java.time.YearMonth;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author liang.cheng
 * @Date 2023/11/16 10:43 AM
 * @Description: TODO
 * @Version 1.0
 */
public class ReportCollectServiceImpl extends Service implements ReportCollectService {

    @Override
    public TendencyReportVO getPersonTendency(TendencyReportParam tendencyReportParam) {

        List<ReportColumns> columns = getPTReportColumns();
        List<Integer> deptList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select a.deptid from hrmdepartmentdefined a left join hrmdepartment b on a.deptid =b.id where a.bmcj in (0,1)" +
                " and (b.canceled = 0 or b.canceled is null)");
        while (rs.next()) {
            deptList.add(Util.getIntValue(rs.getString("deptid")));
        }
        String deptIds = tendencyReportParam.getDeptIds();
        if (StringUtils.isNotEmpty(deptIds)) {
            List<Integer> list = Arrays.stream(deptIds.split(","))
                    .map(Integer::parseInt)
                    .collect(Collectors.toList());
            deptList = list.stream()
                    .filter(deptList::contains)
                    .collect(Collectors.toList());
        }

        List<ReportChildrenData> datas = new ArrayList<>();
        List<List<Integer>> countLists = new ArrayList<>();

        deptList.forEach(id -> {
            List<Integer> countList = new ArrayList<>();
            ReportChildrenData childrenData = getReportChildrenData(tendencyReportParam,id,countList);
            datas.add(childrenData);
            countLists.add(countList);
        });

        //求和
        List<Integer> result = summary(countLists);

        List<String> stringList = result.stream()
                .map(String::valueOf)
                .collect(Collectors.toList());

        List<String> sums = new ArrayList<>();
        sums.add("总数");
        sums.addAll(stringList);

        datas.add(ReportChildrenData.builder().data(sums).build());
        return TendencyReportVO.builder()
                .columns(columns)
                .datas(datas)
                .build();
    }

    @Override
    public TendencyReportVO getOnJobStatistics() {
        List<ReportColumns> columns = getOJSReportColumns();
        List<Integer> deptList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select a.deptid from hrmdepartmentdefined a left join hrmdepartment b on a.deptid =b.id where a.bmcj in (0,1)" +
                " and (b.canceled = 0 or b.canceled is null)");
        while (rs.next()) {
            deptList.add(Util.getIntValue(rs.getString("deptid")));
        }


        List<ReportChildrenData> datas = new ArrayList<>();
        List<List<Integer>> countLists = new ArrayList<>();

        deptList.forEach(id -> {
            List<Integer> countList = new ArrayList<>();
            ReportChildrenData childrenData = getOnJobChildrenData(id,countList);
            datas.add(childrenData);
            countLists.add(countList);
        });

        //求和
        List<Integer> result = summary(countLists);

        List<String> stringList = result.stream()
                .map(String::valueOf)
                .collect(Collectors.toList());

        List<String> sums = new ArrayList<>();
        sums.add("/");
        sums.add("全公司");
        sums.addAll(stringList);

        datas.add(ReportChildrenData.builder().data(sums).build());
        return TendencyReportVO.builder()
                .columns(columns)
                .datas(datas)
                .build();
    }

    @Override
    public TendencyReportVO getPersonnelSize(PersonnelSizeTopParam param) {
        List<ReportColumns> columns = getPSReportColumns();
        List<String> addList = new ArrayList<>();
        List<String> addRate = new ArrayList<>();
        List<ReportChildrenData> datas = getPersonnelSizeData(param,addList,addRate);
        return TendencyReportVO.builder()
                .columns(columns)
                .datas(datas)
                .build();
    }

    @Override
    public Map<String, Object> getTop(PersonnelSizeTopParam param) {
        Map<String,Object> result = new HashMap<>(4);
        List<String> addList = new ArrayList<>();
        List<String> addRate = new ArrayList<>();
        SubCompanyComInfo comInfo = new SubCompanyComInfo();
        ArrayList compList = new ArrayList<>();
        if (param.getCompany() != null) {
            comInfo.getSubCompanyLists(String.valueOf(param.getCompany()),compList);
            compList.add(param.getCompany());
        }

        if (param.getYear() == null) {
            param.setYear(Year.now().getValue());
        }
        getPersonnelSizeData(param,addList,addRate);
        List<PersonnelSizeTop> pstList = new ArrayList<>();

        RecordSet rs = new RecordSet();
        String startYear = CommonDateUtil.getYearDayFirst(param.getYear());
        String endYear = CommonDateUtil.getYearDayEnd(param.getYear());
        int sumStart = sumsByFirstDate(startYear,compList);
        int sumEnd = sumsByFirstDate(endYear,compList);
        int entrySum = sumsByTwoDate(startYear,endYear,compList);
        int leaveSum = 0;

        rs.executeQuery("select count(1) as leavesum from hrmstatushistory where type_n = 5 and changedate >= ? and changedate <= ?",startYear,endYear);
        if (rs.next()) {
            leaveSum = Util.getIntValue(rs.getString("leavesum"));
        }
        int addPerson = entrySum - leaveSum;

        String rate = calculateRate(addRate,addPerson,sumStart);

        pstList.add(PersonnelSizeTop.builder().title("年初人数").count(sumStart).color("#67c29a").build());
        pstList.add(PersonnelSizeTop.builder().title("累计入职人数").count(entrySum).color("#9c5113").build());
        pstList.add(PersonnelSizeTop.builder().title("累计离职人数").count(leaveSum).color("#e26b0a").build());
        pstList.add(PersonnelSizeTop.builder().title("年末人数").count(sumEnd).color("#35889c").build());
        pstList.add(PersonnelSizeTop.builder().title("净增长率").count(rate).color("#f6d262").build());
        result.put("pstTop",pstList);
        result.put("list",addList);
        result.put("rate",addRate);
        return result;
    }

    private List<ReportChildrenData> getPersonnelSizeData(PersonnelSizeTopParam param, List<String> addList, List<String> addRate) {
        RecordSet rs = new RecordSet();
        List<ReportChildrenData> result = new ArrayList<>();
        if (param.getYear() == null) {
            param.setYear(Year.now().getValue());
        }
        SubCompanyComInfo comInfo = new SubCompanyComInfo();
        ArrayList compList = new ArrayList<>();
        if (param.getCompany() != null) {
            comInfo.getSubCompanyLists(String.valueOf(param.getCompany()),compList);
            compList.add(param.getCompany());
        }

        List<YearMonth> yearMonths = CommonDateUtil.getYearMonths(param.getYear());
        for (int i = 0; i < yearMonths.size(); i++) {
            List<String> data = new ArrayList<>();
            String startMonth = CommonDateUtil.getFormatYear(CommonDateUtil.toDateStartOfMonth(yearMonths.get(i)));
            String endMonth = CommonDateUtil.getFormatYear(CommonDateUtil.toDateEndOfMonth(yearMonths.get(i)));
            int sumStart = sumsByFirstDate(startMonth,compList);
            int sumEnd = sumsByFirstDate(endMonth,compList);
            int entrySum = sumsByTwoDate(startMonth,endMonth,compList);
            int leaveSum = 0;
            data.add(String.format("%d 月",i+1));
            rs.executeQuery("select count(1) as leavesum from hrmstatushistory where type_n = 5 and changedate >= ? and changedate <= ?",startMonth,endMonth);
            if (rs.next()) {
                leaveSum = Util.getIntValue(rs.getString("leavesum"));
            }
            int addPerson = entrySum - leaveSum;

            String rate = calculateRate(addRate,addPerson,sumStart);
            addList.add(String.valueOf(addPerson));

            data.add(String.valueOf(sumStart));
            data.add(String.valueOf(entrySum));
            data.add(String.valueOf(leaveSum));
            data.add(String.valueOf(addPerson));
            data.add(String.valueOf(sumEnd));
            data.add(rate);
            result.add(ReportChildrenData.builder().data(data).build());
        }

        return result;


    }


    private List<ReportColumns> getPSReportColumns() {
        List<ReportColumns> columns = new ArrayList<>();
        columns.add(ReportColumns.builder().title("月份").key("0").build());
        columns.add(ReportColumns.builder().title("期初人数").key("1").build());
        columns.add(ReportColumns.builder().title("入职人数").key("2").build());
        columns.add(ReportColumns.builder().title("离职人数").key("3").build());
        columns.add(ReportColumns.builder().title("净增长人数").key("4").build());
        columns.add(ReportColumns.builder().title("期末人数").key("5").build());
        columns.add(ReportColumns.builder().title("净增长率").key("6").build());
        return columns;
    }

    @SneakyThrows
    private ReportChildrenData getOnJobChildrenData(Integer id, List<Integer> countList) {
        RecordSet rs = new RecordSet();
        DepartmentComInfo departmentComInfo = new DepartmentComInfo();
        List<String> data = new ArrayList<>();
        data.add(departmentComInfo.getDepartmentCode(String.valueOf(id)));
        data.add(departmentComInfo.getDepartmentName(String.valueOf(id)));
        ArrayList<String> allSubDepartment = new ArrayList<>();
        departmentComInfo.getAllChildDeptByDepId(allSubDepartment, String.valueOf(id));
        allSubDepartment.add(String.valueOf(id));
        String allId = StringUtils.join(allSubDepartment, ",");
        List<YearMonth> yearMonths = CommonDateUtil.getYearMonths(LocalDate.now());
        StringBuilder st = new StringBuilder();
        st.append("select count(1) as sum from hrmresource where status < 4  and companystartdate <= ? and departmentid in");
        st.append(" (").append(allId).append(")");
        yearMonths.forEach(yearMonth -> {
            String endMonth = CommonDateUtil.getFormatYear(CommonDateUtil.toDateEndOfMonth(yearMonth));
            rs.executeQuery(st.toString(),endMonth);
            if (rs.next()) {
                Integer sum = Util.getIntValue(rs.getString("sum"),0);
                data.add(String.valueOf(sum));
                countList.add(sum);
            }
        });
        return ReportChildrenData.builder().data(data).build();
    }


    @SneakyThrows
    private ReportChildrenData getReportChildrenData(TendencyReportParam param, Integer id,List<Integer> countList) {
        RecordSet rs = new RecordSet();
        DepartmentComInfo departmentComInfo = new DepartmentComInfo();
        List<String> data = new ArrayList<>();
        data.add(departmentComInfo.getDepartmentName(String.valueOf(id)));
        ArrayList<String> allSubDepartment = new ArrayList<>();
        departmentComInfo.getAllChildDeptByDepId(allSubDepartment, String.valueOf(id));
        allSubDepartment.add(String.valueOf(id));
        String allId = StringUtils.join(allSubDepartment, ",");

        if (param.getYear() == null) {
            param.setYear(Year.now().getValue());
        }

        String startDate = CommonDateUtil.getYearDayFirst(param.getYear());
        List<YearMonth> yearMonths = CommonDateUtil.getYearMonths(param.getYear());

        StringBuilder st = new StringBuilder();
        st.append("select count(1) as sum from hrmresource where status < 4  and companystartdate >= ? and companystartdate <= ? and departmentid in");
        st.append(" (").append(allId).append(")");
        if (StringUtils.isNotEmpty(param.getUseKind())) {
            st.append(" and usekind = ").append(param.getUseKind());
        }
        yearMonths.forEach(yearMonth -> {
            String endMonth = CommonDateUtil.getFormatYear(CommonDateUtil.toDateEndOfMonth(yearMonth));
            rs.executeQuery(st.toString(),startDate,endMonth);
            if (rs.next()) {
                Integer sum = Util.getIntValue(rs.getString("sum"),0);
                data.add(String.valueOf(sum));
                countList.add(sum);
            }
        });

        return ReportChildrenData.builder().data(data).build();
    }

    /**
     * 单日期
     */
    private int sumsByFirstDate(String firstDate,ArrayList compList) {
        RecordSet rs = new RecordSet();
        StringBuilder sb = new StringBuilder();
        sb.append("select count(1) as sums from hrmresource where status < 4 and companystartdate <= ?");
        if (CollectionUtils.isNotEmpty(compList)) {
            sb.append(" and subcompanyid1 in(").append(StringUtils.join(compList,",")).append(")");
        }
        rs.executeQuery(sb.toString(),firstDate);
        rs.next();
        return Util.getIntValue(rs.getString("sums"));

    }

    /**
     * 双日期
     */
    private int sumsByTwoDate(String firstDate,String twoDate,ArrayList compList) {
        RecordSet rs = new RecordSet();
        StringBuilder sb = new StringBuilder();
        sb.append("select count(1) as sums from hrmresource where companystartdate >= ? and companystartdate <= ?");
        if (CollectionUtils.isNotEmpty(compList)) {
            sb.append(" and subcompanyid1 in(").append(StringUtils.join(compList,",")).append(")");
        }
        rs.executeQuery(sb.toString(),firstDate,twoDate);
        rs.next();
        return Util.getIntValue(rs.getString("sums"));

    }



    /**
     * 月份1
     */
    private List<ReportColumns> getPTReportColumns() {
        List<ReportColumns> columns = new ArrayList<>();
        columns.add(ReportColumns.builder().title("月份").key("0").build());
        columns.add(ReportColumns.builder().title("1月").key("1").build());
        columns.add(ReportColumns.builder().title("2月").key("2").build());
        columns.add(ReportColumns.builder().title("3月").key("3").build());
        columns.add(ReportColumns.builder().title("4月").key("4").build());
        columns.add(ReportColumns.builder().title("5月").key("5").build());
        columns.add(ReportColumns.builder().title("6月").key("6").build());
        columns.add(ReportColumns.builder().title("7月").key("7").build());
        columns.add(ReportColumns.builder().title("8月").key("8").build());
        columns.add(ReportColumns.builder().title("9月").key("9").build());
        columns.add(ReportColumns.builder().title("10月").key("10").build());
        columns.add(ReportColumns.builder().title("11月").key("11").build());
        columns.add(ReportColumns.builder().title("12月").key("12").build());
        return columns;
    }

    /**
     * 月份2
     */
    private List<ReportColumns> getOJSReportColumns() {
        List<ReportColumns> columns = new ArrayList<>();
        columns.add(ReportColumns.builder().title("部门编号").key("-1").build());
        columns.add(ReportColumns.builder().title("部门名称").key("0").build());
        columns.add(ReportColumns.builder().title("一月").key("1").build());
        columns.add(ReportColumns.builder().title("二月").key("2").build());
        columns.add(ReportColumns.builder().title("三月").key("3").build());
        columns.add(ReportColumns.builder().title("四月").key("4").build());
        columns.add(ReportColumns.builder().title("五月").key("5").build());
        columns.add(ReportColumns.builder().title("六月").key("6").build());
        columns.add(ReportColumns.builder().title("七月").key("7").build());
        columns.add(ReportColumns.builder().title("八月").key("8").build());
        columns.add(ReportColumns.builder().title("九月").key("9").build());
        columns.add(ReportColumns.builder().title("十月").key("10").build());
        columns.add(ReportColumns.builder().title("十一月").key("11").build());
        columns.add(ReportColumns.builder().title("十二月").key("12").build());
        return columns;
    }

    /**
     * 求和
     */
    private List<Integer> summary(List<List<Integer>> countLists){
        return countLists.stream()
                .reduce((a, b) -> {
                    List<Integer> sum = new ArrayList<>();
                    for (int i = 0; i < a.size(); i++) {
                        sum.add(a.get(i) + b.get(i));
                    }
                    return sum;
                })
                .orElse(new ArrayList<>());
    }

    /**
     * 求比例
     */
    private String calculateRate(List<String> addRate,int add,int sum) {
        String rate;
        if (sum != 0) {
            double rt = (double) add * 100 / sum;
            DecimalFormat decimalFormat = new DecimalFormat("0.00");
            decimalFormat.setRoundingMode(java.math.RoundingMode.HALF_UP);
            rate = decimalFormat.format(rt) + "%";
            addRate.add(decimalFormat.format(rt));
        }else {
            rate = String.valueOf(add * 100) +"%";
            addRate.add(String.valueOf(add * 100));
        }

        return rate;
    }


}
