package com.engine.thinktrans.service.impl;

import com.engine.common.util.ServiceUtil;
import com.engine.core.impl.Service;
import com.engine.thinktrans.entity.ReportChildrenData;
import com.engine.thinktrans.entity.ReportColumns;
import com.engine.thinktrans.entity.param.PersonnelSizeTopParam;
import com.engine.thinktrans.entity.param.TendencyReportParam;
import com.engine.thinktrans.entity.vo.TendencyReportVO;
import com.engine.thinktrans.service.ReportCollectService;
import com.engine.thinktrans.service.ReportExportService;
import com.engine.thinktrans.util.ExcelUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import weaver.hrm.User;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author liang.cheng
 * @Date 2023/11/17 2:04 PM
 * @Description: TODO
 * @Version 1.0
 */
public class ReportExportServiceImpl extends Service implements ReportExportService {

    private ReportCollectService getReportCollectService(User user) {
        return ServiceUtil.getService(ReportCollectServiceImpl.class,user);
    }

    @Override
    public XSSFWorkbook personTendencyExport(TendencyReportParam tendencyReportParam) {

        TendencyReportVO personTendency = getReportCollectService(user).getPersonTendency(tendencyReportParam);

        // 1.工作簿名称
        String sheetName = "年度人力趋势("+tendencyReportParam.getYear()+"年)";
        // 2.表头(后面动态获取)
        List<List<Object>> excelSheetData = new ArrayList<>();

        List<ReportColumns> columns = personTendency.getColumns();
        String[] header = columns.stream()
                .map(ReportColumns::getTitle)
                .toArray(String[]::new);

        excelSheetData.add(Arrays.asList(header));

        //数据
        List<ReportChildrenData> datas = personTendency.getDatas();
        List<List<Object>> rows = datas.stream()
                .map(data -> (List<Object>) new ArrayList<Object>(data.getData()))
                .collect(Collectors.toList());


        excelSheetData.addAll(rows);
        return ExcelUtil.genWorkbook(excelSheetData, sheetName);
    }

    @Override
    public XSSFWorkbook onJobStatisticsExport() {
        TendencyReportVO personTendency = getReportCollectService(user).getOnJobStatistics();

        // 1.工作簿名称
        String sheetName = "实时在职人数";
        // 2.表头(后面动态获取)
        List<List<Object>> excelSheetData = new ArrayList<>();

        List<ReportColumns> columns = personTendency.getColumns();
        String[] header = columns.stream()
                .map(ReportColumns::getTitle)
                .toArray(String[]::new);

        excelSheetData.add(Arrays.asList(header));

        //数据
        List<ReportChildrenData> datas = personTendency.getDatas();
        List<List<Object>> rows = datas.stream()
                .map(data -> (List<Object>) new ArrayList<Object>(data.getData()))
                .collect(Collectors.toList());


        excelSheetData.addAll(rows);
        return ExcelUtil.genWorkbook(excelSheetData, sheetName);
    }

    @Override
    public XSSFWorkbook personnelSizeExport(PersonnelSizeTopParam param) {
        TendencyReportVO personTendency = getReportCollectService(user).getPersonnelSize(param);

        // 1.工作簿名称
        String sheetName = "公司人员规模分析";
        // 2.表头(后面动态获取)
        List<List<Object>> excelSheetData = new ArrayList<>();

        List<ReportColumns> columns = personTendency.getColumns();
        String[] header = columns.stream()
                .map(ReportColumns::getTitle)
                .toArray(String[]::new);

        excelSheetData.add(Arrays.asList(header));

        //数据
        List<ReportChildrenData> datas = personTendency.getDatas();
        List<List<Object>> rows = datas.stream()
                .map(data -> (List<Object>) new ArrayList<Object>(data.getData()))
                .collect(Collectors.toList());

        excelSheetData.addAll(rows);
        return ExcelUtil.genWorkbook(excelSheetData, sheetName);
    }
}
