package com.engine.thinktrans.web;

import com.engine.common.util.ServiceUtil;
import com.engine.thinktrans.entity.param.PersonnelSizeTopParam;
import com.engine.thinktrans.entity.param.TendencyReportParam;
import com.engine.thinktrans.service.ReportExportService;
import com.engine.thinktrans.service.impl.ReportExportServiceImpl;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import weaver.hrm.HrmUserVarify;
import weaver.hrm.User;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.StreamingOutput;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.time.LocalDate;

/**
 * @Author liang.cheng
 * @Date 2023/11/17 1:57 PM
 * @Description: TODO
 * @Version 1.0
 */
public class ReportExportController {

    private ReportExportService getReportExportService(User user) {
        return ServiceUtil.getService(ReportExportServiceImpl.class,user);
    }

    @GET
    @Path("/personTendency/export")
    @Produces(MediaType.APPLICATION_OCTET_STREAM)
    public Response personTendencyExport(@Context HttpServletRequest request, @Context HttpServletResponse response,
                                         @QueryParam("deptIds") String deptIds, @QueryParam("year") Integer year, @QueryParam("useKind") String useKind) {
        User user = HrmUserVarify.getUser(request, response);
        TendencyReportParam tendencyReportParam = TendencyReportParam.builder().deptIds(deptIds).year(year).useKind(useKind).build();
        XSSFWorkbook workbook = getReportExportService(user).personTendencyExport(tendencyReportParam);
        String time = LocalDate.now().toString();
        String fileName = "年度人力趋势报表" + time;
        try {
            fileName = URLEncoder.encode(fileName + ".xlsx", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        StreamingOutput output = outputStream -> {
            workbook.write(outputStream);
            outputStream.flush();
        };
        response.setContentType("application/octet-stream");
        return Response.ok(output).header("Content-disposition", "attachment;filename=" + fileName).header("Cache-Control", "no-cache").build();
    }

    @GET
    @Path("/onJobStatistics/export")
    @Produces(MediaType.APPLICATION_OCTET_STREAM)
    public Response onJobStatisticsExport(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        User user = HrmUserVarify.getUser(request, response);
        XSSFWorkbook workbook = getReportExportService(user).onJobStatisticsExport();
        String time = LocalDate.now().toString();
        String fileName = "实时在职人数报表" + time;
        try {
            fileName = URLEncoder.encode(fileName + ".xlsx", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        StreamingOutput output = outputStream -> {
            workbook.write(outputStream);
            outputStream.flush();
        };
        response.setContentType("application/octet-stream");
        return Response.ok(output).header("Content-disposition", "attachment;filename=" + fileName).header("Cache-Control", "no-cache").build();
    }

    @GET
    @Path("/personnelSize/export")
    @Produces(MediaType.APPLICATION_OCTET_STREAM)
    public Response personnelSizeExport(@Context HttpServletRequest request, @Context HttpServletResponse response,@QueryParam("year") Integer year,@QueryParam("company") Integer company) {
        User user = HrmUserVarify.getUser(request, response);
        PersonnelSizeTopParam build = PersonnelSizeTopParam.builder().year(year).company(company).build();
        XSSFWorkbook workbook = getReportExportService(user).personnelSizeExport(build);
        String time = LocalDate.now().toString();
        String fileName = "公司人员规模报表" + time;
        try {
            fileName = URLEncoder.encode(fileName + ".xlsx", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        StreamingOutput output = outputStream -> {
            workbook.write(outputStream);
            outputStream.flush();
        };
        response.setContentType("application/octet-stream");
        return Response.ok(output).header("Content-disposition", "attachment;filename=" + fileName).header("Cache-Control", "no-cache").build();
    }
}
